#
# This is the SPEC file for creating binary and source RPMs for the VMs.
#
#
# The Qubes OS Project, http://www.qubes-os.org
#
# Copyright (C) 2010  Joanna Rutkowska <joanna@invisiblethingslab.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#


# default value in case of no qubes-builder's one

Name:		qubes-gui-agent
Version:	4.2.9
Release:	1%{dist}
Summary:	The Qubes GUI Agent for AppVMs

Group:		Qubes
Vendor:		Invisible Things Lab
License:	GPL
URL:		https://www.qubes-os.org


%define pa_ver_full %((pkg-config --modversion libpulse 2>/dev/null || echo 0.0) | cut -d "-" -f 1)
%define pa_module_dir %(pkg-config --variable=modlibexecdir libpulse)

BuildRequires:	gcc
BuildRequires:	libX11-devel
BuildRequires:	libXcomposite-devel
BuildRequires:	libXdamage-devel
BuildRequires:	libXfixes-devel
BuildRequires:	libXt-devel
BuildRequires:	libtool-ltdl-devel
BuildRequires:	libtool
BuildRequires:	pulseaudio-libs-devel >= 0.9.21, pulseaudio-libs-devel <= 16.99.1
BuildRequires:	xorg-x11-server-devel
BuildRequires:	pam-devel
BuildRequires:	qubes-libvchan-devel
BuildRequires:	qubes-gui-common-devel >= 4.1.0
BuildRequires:	qubes-db-devel
BuildRequires:	xen-devel
BuildRequires:	mesa-libgbm-devel
BuildRequires:	systemd-rpm-macros
Requires:	Xorg %(xserver-sdk-abi-requires ansic)
Requires:	Xorg %(xserver-sdk-abi-requires videodrv)
Requires:	libXfixes
Requires:	qubes-core-vm >= 3.0.14
Requires:	xorg-x11-xinit
# qubesdb-read --wait option
Requires:	qubes-db >= 4.1.4
Requires:	python%{python3_pkgversion}-xcffib
Requires:   xorg-x11-server-Xorg
Requires:   xorg-x11-server-Xephyr
Requires:   setxkbmap
Requires:   xsetroot
Requires:   xrdb
Requires:   xmodmap
Requires:   xev
Requires:   xdpyinfo
Requires:   xprop
Requires:   xrandr

Provides:   qubes-gui-vm = %{version}-%{release}
Obsoletes:  qubes-gui-vm < 4.0.0

Source0: %{name}-%{version}.tar.gz

Requires: (%{name}-selinux if selinux-policy)

%package -n pulseaudio-qubes
Summary: Audio support for Qubes VM
# The vchan sink needs .h files from pulseaudio sources
# that are not exported by any *-devel packages; thus they are internal and
# possible to change across version. They are copied to gui git.
# It is possible that our code will work fine with any later pulseaudio
# version; but this needs to be verified for each pulseaudio version.
Requires:	pulseaudio = %{pa_ver_full}
# Needed so that systemd can manage PulseAudio
Conflicts:	qubes-core-agent < 4.2.5
Conflicts:  qubes-gui-vm < 4.0.0

%description -n pulseaudio-qubes
Pulseaudio module to enable sound support in Qubes VM

%package -n pipewire-qubes
Summary:       Experimental PipeWire audio support for Qubes VMs
Requires:      pipewire
BuildRequires: pipewire-devel
BuildRequires: qubes-db-devel
BuildRequires: qubes-libvchan-devel

%description -n pipewire-qubes
Experimental PipeWire module to enable sound support in Qubes VMs.
Unlike the PulseAudio support in the pulseaudio-qubes package, this
works with the stock PipeWire service and does not depend on internal
PipeWire headers.  It also promises lower latency than PulseAudio,
at least once the audio daemon supports low latency.

%package xfce
Summary: XFCE desktop support for Qubes VM

%description xfce
XFCE desktop support for Qubes VM

%package -n qubes-gui-vnc
Summary: Remote GuiVM support using VNC
Requires: x11vnc

%description -n qubes-gui-vnc
Remote GuiVM support using VNC

%description
The Qubes GUI agent that needs to be installed in VM in order to provide the
Qubes fancy GUI.

%prep
%setup -q

rm -f pulse/pulsecore
ln -s pulsecore-%{pa_ver_full} pulse/pulsecore

%build
%{?set_build_flags}
#make clean
make appvm

%install
rm -rf -- "$RPM_BUILD_ROOT"
make install "DESTDIR=$RPM_BUILD_ROOT" \
                     LIBDIR=%{_libdir} \
                     DATADIR=%{_datadir} \
                     SYSLIBDIR=/usr/lib \
                     PA_VER_FULL=%{pa_ver_full} \
                     ENABLE_SELINUX=%{with selinux} \
                     UNITDIR=%{_unitdir} \
                     USERUNITDIR=%{_userunitdir} \
                     USERPRESETDIR=%{_userpresetdir} \
                     UDEVRULESDIR=%{_udevrulesdir}

%post
if [ -x /bin/systemctl ] && readlink /sbin/init | grep -q systemd; then
    /bin/systemctl enable qubes-gui-agent.service 2> /dev/null
fi

%post -n pulseaudio-qubes

sed -i '/^autospawn/d' /etc/pulse/client.conf
echo autospawn=no >> /etc/pulse/client.conf

# simplified version of preset_units() from core-agent-linux
preset_units() {
    local represet=
    cat "$1" | while read action unit_name
    do
        if [ "$action" = "#" -a "$unit_name" = "Units below this line will be re-preset on package upgrade" ]
        then
            represet=1
            continue
        fi
        echo "$action $unit_name" | grep -q '^[[:space:]]*[^#;]' || continue
        [ -n "$action" -a -n "$unit_name" ] || continue
        if [ "$2" = "initial" -o "$represet" = "1" ]
        then
            systemctl --no-reload --global preset "$unit_name" >/dev/null 2>&1 || :
        fi
    done
}

if [ $1 -eq 1 ]
then
    preset_units /usr/lib/systemd/user-preset/75-pulseaudio-qubes.preset initial
else
    preset_units /usr/lib/systemd/user-preset/75-pulseaudio-qubes.preset upgrade
fi

%postun -n pulseaudio-qubes

for unit_name in pipewire.socket pipewire.service \
                 pipewire-pulse.socket pipewire-pulse.service \
                 wireplumber.service; do
    systemctl --no-reload --global preset "$unit_name" >/dev/null 2>&1 || :
done

%post -n pipewire-qubes
%systemd_user_post pipewire.service pipewire.socket wireplumber.service

%preun -n pipewire-qubes
%systemd_user_preun pipewire.service pipewire.socket wireplumber.service

%preun
if [ "$1" = 0 ] ; then
	chkconfig qubes-gui-agent off
    [ -x /bin/systemctl ] && /bin/systemctl disable qubes-gui-agent.service
    /usr/bin/glib-compile-schemas %{_datadir}/glib-2.0/schemas &> /dev/null || :
fi

%posttrans
    /usr/bin/glib-compile-schemas %{_datadir}/glib-2.0/schemas &> /dev/null || :

if [ -d '/etc/init.d' ] && ! [ "$(ls -A /etc/init.d/ 2> /dev/null)" ]; then
    rmdir /etc/init.d
fi

%triggerin -n pulseaudio-qubes -- pulseaudio-libs

sed -i '/^autospawn/d' /etc/pulse/client.conf
echo autospawn=no >> /etc/pulse/client.conf

%clean
rm -rf $RPM_BUILD_ROOT
rm -f %{name}-%{version}

%files
%defattr(-,root,root,-)
%_bindir/qubes-gui
%_bindir/qubes-gui-runuser
%_bindir/qubes-session
%_bindir/qubes-run-xorg
%_bindir/qubes-run-xephyr
%_bindir/qubes-start-xephyr
%_bindir/qubes-change-keyboard-layout
%_bindir/qubes-set-monitor-layout
%_libdir/xorg/modules/drivers/qubes_drv.so
%_libdir/xorg/modules/drivers/dummyqbs_drv.so
%_libdir/libxf86-qubes-common.so
%attr(0644,root,root) /etc/X11/xorg-qubes.conf.template
/etc/profile.d/qubes-gui.sh
/etc/profile.d/qubes-gui.csh
%config /etc/pam.d/qubes-gui-agent
%config /etc/security/limits.d/90-qubes-gui.conf
%config /etc/xdg/Trolltech.conf
/etc/X11/xinit/xinitrc.d/20qt-x11-no-mitshm.sh
/etc/X11/xinit/xinitrc.d/20qt-gnome-desktop-session-id.sh
/etc/X11/xinit/xinitrc.d/50guivm-windows-prefix.sh
/etc/qubes-rpc/qubes.SetMonitorLayout
/etc/qubes-rpc/qubes.GuiVMSession
/etc/qubes/post-install.d/20-qubes-guivm-gui-agent.sh
%config /etc/sysconfig/desktop
%_unitdir/qubes-gui-agent.service
%_udevrulesdir/70-master-of-seat.rules
%_sysctldir/30-qubes-gui-agent.conf
/usr/lib/qubes/gtk4-workarounds.sh
/usr/lib/qubes/qubes-gui-agent-pre.sh
/usr/lib/qubes/qubes-keymap.sh
/usr/lib/qubes/qubes-xorg-wrapper
%{_datadir}/glib-2.0/schemas/20_qubes-gui-vm.gschema.override
/usr/lib/qubes/icon-sender
/etc/xdg/autostart/qubes-gtk4-workarounds.desktop
/etc/xdg/autostart/qubes-icon-sender.desktop
/etc/xdg/autostart/qubes-qrexec-fork-server.desktop
/etc/xdg/autostart/qubes-keymap.desktop
%dir /var/log/qubes

%files -n pulseaudio-qubes
%_tmpfilesdir/qubes-pulseaudio.conf
/etc/pulse/qubes-default.pa
%_bindir/start-pulseaudio-with-vchan
%{pa_module_dir}/module-vchan-sink.so
/etc/xdg/autostart/qubes-pulseaudio.desktop
/usr/lib/systemd/user-preset/75-pulseaudio-qubes.preset
/usr/lib/systemd/user/pulseaudio.service.d/30_qubes.conf

%files -n pipewire-qubes
%_datadir/pipewire/pipewire.conf.d/30_qubes.conf
%_userpresetdir/74-qubes-vm.preset
%_libdir/pipewire-0.3/libpipewire-module-qubes.so
%_defaultlicensedir/pipewire-qubes/COPYING
/etc/qubes/post-install.d/20-qubes-pipewire.sh

# Works with ordinary PipeWire service :)

%files xfce
/etc/X11/xinit/xinitrc.d/60xfce-desktop.sh

%files -n qubes-gui-vnc
#/lib/systemd/system/qubes-gui-vncserver.service
%attr(0644,root,root) /etc/X11/xorg-qubes-x11vnc.conf.template
%attr(0644,root,root) /etc/systemd/system/lightdm.service.d/qubes-guivm-vnc.conf
%_bindir/qubes-run-x11vnc

%changelog
* Sun Sep 10 2023 Qubes OS Team <qubes-devel@googlegroups.com>
- For complete changelog see: https://github.com/QubesOS/qubes-gui-agent-linux

* Sun Sep 10 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - b2ff128
- version 4.2.9

* Thu Sep 07 2023 Frédéric Pierret (fepitre) <frederic@invisiblethingslab.com> - 2b14649
- Update for Fedora 39

* Sat Sep 02 2023 fepitre-bot <fepitre-bot@qubes-os.org> - bfd9e49
- Add pulseaudio-16.99.1 headers

* Thu Aug 31 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 52facec
- qubes-gui-runuser: obtain default user from qubesdb

* Thu Aug 31 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - fde57fa
- Fix unchecked malloc()

* Thu Aug 31 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 6737a90
- Avoid reserved identifier violation

* Thu Aug 31 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 802e63a
- fix some clang warnings

* Thu Aug 31 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 42bf492
- Always see if module-vchan-sink.so must be rebuilt

* Thu Aug 31 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - ef1ead4
- Switch from select(2) to poll(2).

* Mon Aug 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - d475a29
- Ban old-style declarations and definitions

* Mon Aug 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 0dad6f0
- Remove obsolete checks for disposable VM resume

* Mon Aug 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - a12e3e6
- Update .gitignore files

* Mon Aug 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 9056a41
- Fix missing prototypes

* Mon Aug 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - b724611
- Ensure that all function declarations have prototypes

* Mon Aug 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 8703c06
- Consolodate gui-common/ and common/ into gui-agent/

* Wed Aug 09 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 6d33129
- Set XDG_CURRENT_DESKTOP

* Wed Aug 09 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 76859c3
- Protect against more unlikely errors

* Wed Aug 09 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 6e17419
- Set XDG_SESSION_DESKTOP

* Wed Aug 09 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 042bbb9
- Add XDG_SESSION_TYPE support

* Wed Aug 09 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 2a49f24
- Better error checking

* Fri Jul 21 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - e38b067
- version 4.2.8

* Fri Jul 21 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - fe3546a
- Fix mode of 75-pulseaudio-qubes.preset

* Fri Jul 21 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 1b332db
- Remove trailing space in 75-pulseaudio-qubes.preset

* Fri Jul 21 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - cf8328e
- rpm: restore pipewire units on pulseaudio-qubes removal

* Fri Jul 21 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - aa43702
- pulseaudio: disable pipewire-pulse too

* Fri Jul 21 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - f4ea6d1
- debian: preset user services on pulseaudio-qubes install/uninstall

* Wed Jul 12 2023 Solar Designer <solar@openwall.com> - f3f6886
- qubes-keymap.sh: Don't add ",us" for layout "si"

* Wed Jul 12 2023 Solar Designer <solar@openwall.com> - 4b6956e
- qubes-keymap.sh: Avoid xkb option accumulation

* Tue Jul 04 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - eebfd81
- Use set -a instead of sed

* Tue Jul 04 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 50cddcc
- Do not hardcode "user" as the GUI user

* Tue Jul 04 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - edfaff3
- Increase locked memory limit for "qubes" group

* Tue Jul 04 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 4d9f915
- Update comment to reflect current GUI agent

* Fri Jun 02 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 9021f67
- Autotools modernization

* Fri Jun 02 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - b44a426
- Support MSG_WINDOW_DUMP_ACK

* Fri Jun 02 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 2d922ce
- Use malloc instead of alloca

* Fri Jun 02 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 93b9edb
- Add simple reference counting of pixmaps

* Fri Jun 02 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - d51099a
- Add a couple of assertions

* Fri Jun 02 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - b9ecef8
- Initial protocol 1.7 support

* Fri Jun 02 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 85e878d
- Switch to protocol 1.4

* Sun May 28 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - e11a0bc
- ci: extend exclude list

* Sat May 27 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - dce4395
- ci: use CI_BRANCH_REF

* Wed May 24 2023 Augsch123 <130238014+Augsch123@users.noreply.github.com> - cc4d4fa
- Add libpam-systemd dependency

* Tue May 23 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - d3ec401
- archlinux: fix install-pipewire call

* Tue May 16 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 9d262c4
- version 4.2.7

* Tue May 16 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 1aad0f6
- deb: Fix clearing source commands

* Tue May 16 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - e81afe3
- version 4.2.6

* Tue May 16 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 644a0b0
- Cleanup unused file

* Mon May 15 2023 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 471c7ac
- Update CI exclude for install

* Mon May 15 2023 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - c4460b7
- pipewire: drop service vm to be set

* Mon May 15 2023 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 3d9b94a
- Step back for jammy on pipewire

